# -*- coding: utf-8 -*- 
# @Time    : 08/04/2024 
# @Author  : nzooherd 
# @File    : video_clean.py 
# @Software: Visual Studio Code

import json
import opencc
import asyncio
from collections import defaultdict
from logging import getLogger
import time
import requests
from sqlalchemy import create_engine, text
from telegram import Video
from bootstrap import BaseApp
from config import Config
from component.pornherd_bot import mybot


class VideoClean(BaseApp):
    logger = getLogger(__name__)

    def __init__(self):
        self.engine = create_engine(Config.SYNC_DATABASE_URL, pool_size=32, max_overflow=64)
        try:
            self.loop = asyncio.get_event_loop()
        except RuntimeError:
            self.loop = asyncio.new_event_loop()
            asyncio.set_event_loop(self.loop)
        self.loop.run_until_complete(self.init())
    
    async def init(self):
        await mybot.init(clients=[mybot.me_client])
        self.WEBPAGE_BOT = await mybot.me_client.resolve_username("WebpageBot")

    def clean_invalid_video(self):
        with self.engine.connect() as conn:
            sql_query = text("select preview_url from video")
            result = conn.execute(sql_query)
            invalid_urls = []
            for row in result:
                if not self._validate_preview_url(row.preview_url):
                    invalid_urls.append(row.preview_url)
            print(f"Invalid video url: {invalid_urls}")
            sql_query = text("delete from video_tags where video_id in (select id from video where preview_url in :invalid_urls)")
            result = conn.execute(sql_query, {'invalid_urls': tuple(invalid_urls)})
            sql_query = text("delete from video where preview_url in :invalid_urls")
            result = conn.execute(sql_query, {'invalid_urls': tuple(invalid_urls)})
            conn.commit()

    def _validate_preview_url(self, url):
        response = requests.head(url)
        content_type = response.headers.get('Content-Type', '')
        if 'xml' in content_type.lower():
            return False
        if 'video' in content_type.lower():
            return True
    
    def inline_preview_url(self):
        with self.engine.connect() as conn:
            sql_query = text("select preview_url from video where status != 1")
            result = conn.execute(sql_query)
            for row in result:
                if not row.preview_url:
                    continue
                self.loop.run_until_complete(mybot.me_client.send_message(self.WEBPAGE_BOT, row.preview_url))
                time.sleep(5)
                conn.execute(text("update video set status = 2 where preview_url = :preview_url"), {'preview_url': row.preview_url})
                conn.commit()

    def update_video_status(self):
        with self.engine.connect() as conn:
            sql_query = text("select * from video where status = 2")
            result = conn.execute(sql_query)
            for row in result:
                if row.name:
                    sql_query = text("update video set status = 1 where id = :id")
                    conn.execute(sql_query, {'id': row.id})
            conn.commit()
    
    def select_tags(self):
        converter = opencc.OpenCC('t2s')
        tags = set()
        video2tags = defaultdict(set)
        with self.engine.connect() as conn:
            sql_query = text("select * from video_tags")
            result = conn.execute(sql_query)
            for row in result:
                video2tags[row.video_id].add(converter.convert(row.tag))
                tags.add(row.tag)
        
        valid_tags = set(['偷窥', '素人', '连裤袜', '亲吻', '美少女', '荡妇', '颜射', '中出', '玩具', '多汗', '放尿', '巨乳', '凌辱', '浴室', '足交', '出轨', '淫语', 'M男', '企划片', '按摩', '恶作剧', '戏剧', '辣妹', '女教师', '窈窕', '过激系', '花痴', '羞耻', '自拍', '受付嬢', '美容院', '恋腿癖', '护士', '潮吹', '不伦', '处男', '巨尻', '女上司', '药物', '滥交', '乳交', '苗条', '多P'])
        for video_id, video_tags in video2tags.items():
            inter_tags = valid_tags & video_tags
            new_video_tags = set(inter_tags)
            #for tag in video_tags:
            #    if tag in inter_tags:
            #        continue
            #    for valid_tag in valid_tags:
            #        if set(tag) & set(valid_tag):
            #            new_video_tags.add(valid_tag)
            video2tags[video_id] = new_video_tags
            print(f"video_id: {video_id}, tags: {video2tags[video_id]}")            
    
    def update_metadata(self):
        with self.engine.connect() as conn:
            sql_query = text("select * from video")
            result = conn.execute(sql_query)
            for row in result:
                if not row.filename:
                    continue
                with open(f"/home/nzooherd/Pornherd/metadata/{row.filename}", "w") as f:
                    f.write("metadata")


if __name__ == "__main__":
    VideoClean().update_video_status()